/* ***********************************************************
 * This file was automatically generated on 2012-09-26.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for controlling a piezo buzzer
	/// </summary>
	public class BrickletPiezoBuzzer : Device 
	{
		private static byte FUNCTION_BEEP = 1;
		private static byte FUNCTION_MORSE_CODE = 2;
		private static byte CALLBACK_BEEP_FINISHED = 3;
		private static byte CALLBACK_MORSE_CODE_FINISHED = 4;


		/// <summary>
		///  This callback is triggered if a beep set by <see cref="Tinkerforge.BrickletPiezoBuzzer.Beep"/> is finished
		/// </summary>
		public delegate void BeepFinished();

		/// <summary>
		///  This callback is triggered if the playback of the morse code set by
		///  <see cref="Tinkerforge.BrickletPiezoBuzzer.MorseCode"/> is finished.
		/// </summary>
		public delegate void MorseCodeFinished();

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickletPiezoBuzzer(string uid) : base(uid) 
		{
			this.expectedName = "Piezo Buzzer Bricklet";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 0;
			messageCallbacks[CALLBACK_BEEP_FINISHED] = new MessageCallback(CallbackBeepFinished);
			messageCallbacks[CALLBACK_MORSE_CODE_FINISHED] = new MessageCallback(CallbackMorseCodeFinished);
		}

		/// <summary>
		///  Beeps with the duration in ms. For example: If you set a value of 1000,
		///  the piezo buzzer will beep for one second.
		/// </summary>
		public void Beep(uint duration)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_BEEP, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(duration, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Sets morse code that will be played by the piezo buzzer. The morse code
		///  is given as a string consisting of "." (dot), "-" (minus) and " " (space)
		///  for *dits*, *dahs* and *pauses*. Every other character is ignored.
		///  
		///  For example: If you set the string "...---...", the piezo buzzer will beep
		///  nine times with the durations "short short short long long long short 
		///  short short".
		///  
		///  The maximum string size is 60.
		/// </summary>
		public void MorseCode(string morse)
		{
			byte[] data_ = new byte[64];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_MORSE_CODE, 1, data_);
			LEConverter.To((ushort)64, 2, data_);
			LEConverter.To(morse, 4, 60, data_);

			SendRequestNoResponse(data_);
		}

		internal int CallbackBeepFinished(byte[] data_)
		{
			((BeepFinished)callbacks[CALLBACK_BEEP_FINISHED])();
			return 4;
		}

		internal int CallbackMorseCodeFinished(byte[] data_)
		{
			((MorseCodeFinished)callbacks[CALLBACK_MORSE_CODE_FINISHED])();
			return 4;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(BeepFinished))
			{
				callbacks[CALLBACK_BEEP_FINISHED] = d;
			}
			else if(d.GetType() == typeof(MorseCodeFinished))
			{
				callbacks[CALLBACK_MORSE_CODE_FINISHED] = d;
			}
		}
	}
}
